unit DbgView;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, BGRABitmap, IndLed, Forms, Controls, Graphics,
  Dialogs, Grids, ExtCtrls, StdCtrls, IniPropStorage, Spin;

type

  { TDbgForm }

  TDbgForm = class(TForm)
    Bevel1: TBevel;
    Bevel2: TBevel;
    Bevel3: TBevel;
    IniPropStorage1: TIniPropStorage;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    ValC: TPaintBox;
    Strip: TPaintBox;
    Panel5: TPanel;
    ValBright: TLabel;
    Panel1: TPanel;
    Panel2: TPanel;
    Panel4: TPanel;
    Pix: TStringGrid;
    ValColor: TLabel;
    ValFade: TLabel;
    Vars: TStringGrid;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure PixDrawCell(Sender: TObject; aCol, aRow: Integer; aRect: TRect;
      aState: TGridDrawState);
    procedure StripCntChange(Sender: TObject);
    procedure StripPaint(Sender: TObject);
    procedure ValCPaint(Sender: TObject);
    procedure VarsDrawCell(Sender: TObject; aCol, aRow: Integer; aRect: TRect;
      aState: TGridDrawState);
  private
    { private declarations }
    Bmp : TBGRABitmap;
    function AddBright(aColor : Tcolor; aBright : integer) : TColor;

  public
    { public declarations }
    procedure UpdateDbg;
    procedure WaitDbg;
  end;

var
  DbgForm: TDbgForm;

implementation
uses
  ScriptEditor, constants, BGRABitmapTypes;
{$R *.frm}



{ TDbgForm }

procedure TDbgForm.FormCreate(Sender: TObject);
var
  i : integer;
begin
  Bmp := TBGRABitmap.Create(Strip.ClientWidth, Strip.ClientHeight);
  Pix.ColWidths[0] := Pix.Width - Pix.DefaultColWidth * (Pix.ColCount-1);
  for i:=1 to cTotalPix do
      Pix.Cells[i, 0] := IntToHex(i-1, 2);
  Pix.Cells[0,1] := 'Bright';
  Pix.Cells[0,2] := 'Fade';
  Pix.Cells[0,3] := 'Color';
  for i:=0 to cTotalVar-1 do
      Vars.Cells[i,0] := 'V' + IntToHex(i, 1);
end;

procedure TDbgForm.FormShow(Sender: TObject);
begin
  UpdateDbg;
end;

procedure DecRect(var aRect : TRect);
begin
  with aRect do begin
    inc(Left); inc(Top);
    dec(Right); dec(Bottom);
  end;
end;

procedure TDbgForm.PixDrawCell(Sender: TObject; aCol, aRow: Integer;
  aRect: TRect; aState: TGridDrawState);
var
  X, Y :integer;
begin
  if (aCol = 0) or (aRow = 0)
     then exit;
  DecRect(aRect);
  Y := aRect.Top + (aRect.Bottom - aRect.Top - Pix.Canvas.TextHeight('00')) div 2;
  X := aRect.Left + (aRect.Right - aRect.Left - Pix.Canvas.TextWidth('00')) div 2;
  if (aRow = 3) then begin
    // Color
    Pix.Canvas.Brush.Color := Script.Pixel[aCol-1].Color;
    Pix.Canvas.Ellipse(aRect);
  end;
  if(aRow = 2) then begin
    // Fade
    Pix.Canvas.TextOut(X, Y, IntToHex(Script.Pixel[aCol-1].Fade, 2));
  end;
  if(aRow = 1) then begin
    // ValBright
    Pix.Canvas.TextOut(X, Y, IntToHex(Script.Pixel[aCol-1].Bright, 2));
  end;
end;

procedure TDbgForm.StripCntChange(Sender: TObject);
begin
  Strip.Repaint;
end;

function TDbgForm.AddBright(aColor : Tcolor; aBright : integer) : TColor;
var
  R, G, B : integer;
begin
  //Result := aColor; exit;
  R := aColor and $FF;
  R := ((R * aBright) shr 8) and $FF;
  G := aColor and $FF00;
  G := (((G shr 8) * aBright) shr 8) and $FF;
  B := aColor and $FF0000;
  B := (((B shr 16) * aBright) shr 8) and $FF;
  Result := R + (G shl 8) + (B shl 16);
end;

procedure TDbgForm.StripPaint(Sender: TObject);
var
  i, n : integer;
  x0, x, y, w : integer;
begin
  with Bmp do begin
     Canvas.Pen.Color := clBlack;
     Canvas.Brush.Color := clBlack;
     Canvas.Rectangle(Strip.ClientRect);
     Canvas.Pen.Color := $202020;
     w := Strip.ClientWidth div cTotalPix;
     y := (Strip.ClientHeight - w) div 2;
     x0 := (Strip.ClientWidth - w * cTotalPix) div 2;
     for i:=0 to cTotalPix-1 do begin
         n := i mod cTotalPix;
         x := x0 + i*w;
         Canvas.Brush.Color := AddBright(Script.Pixel[n].Color, Script.Pixel[n].Bright);
         if w <= 3 then
            Canvas.Pen.Color := Canvas.Brush.Color;
         Canvas.Ellipse(x, y, x + w, y + w);
         //Canvas.Ellipse(x+5, y+5, x + w - 5, y + w - 5);
     end;
  end;
  Bmp.Draw(Strip.Canvas, 0, 0);
end;

procedure TDbgForm.ValCPaint(Sender: TObject);
begin
  ValC.Canvas.Pen.Color := clBlack;
  ValC.Canvas.Brush.Color := Script.CurColor;
  ValC.Canvas.Ellipse(ValC.ClientRect);
end;

procedure TDbgForm.VarsDrawCell(Sender: TObject; aCol, aRow: Integer;
  aRect: TRect; aState: TGridDrawState);
var
  X, Y :integer;
begin
  if(aRow = 1) then begin
    DecRect(aRect);
    Y := aRect.Top + (aRect.Bottom - aRect.Top - Vars.Canvas.TextHeight('00')) div 2;
    X := aRect.Left + (aRect.Right - aRect.Left - Vars.Canvas.TextWidth('00')) div 2;
    Vars.Canvas.TextOut(X, Y, IntToHex(Script.CurV[aCol], 2));
  end;
end;

procedure TDbgForm.UpdateDbg;
begin
  Pix.Repaint;
  Vars.Repaint;
  ValBright.Caption := IntToHex(Script.CurBright, 2);
  ValFade.Caption := IntToHex(Script.CurFade, 2);
  ValColor.Caption := IntToHex(Script.CurColor and $FF, 2) +
                     IntToHex((Script.CurColor and $FF00) shr 8, 2) +
                     IntToHex((Script.CurColor and $FF0000)  shr 16, 2);
  ValC.Repaint;
end;

procedure TDbgForm.WaitDbg;
begin
  Strip.Repaint;
end;

end.

